# coding=utf-8
import os
from django.conf import settings
from django.db import models
from django.template.loaders.app_directories import app_template_dirs
from django.utils.translation import ugettext_lazy as _

try:
    from django.utils.timezone import now
except ImportError:
    from datetime import datetime

    now = datetime.now


def get_template_choices(templates_dir_name='emailtemplates', suffix='.html'):
    """
    returns list of html templates from emailtemplates/*/
    """
    templates_list = []
    for template_dir_path in (settings.TEMPLATE_DIRS + app_template_dirs):
        path = os.path.join(template_dir_path, templates_dir_name)

        for root, dirs, files in os.walk(path):
            for name in files:
                filename = os.path.join(root, name).replace(path, '')
                if filename.endswith(suffix):
                    templates_list.append(filename[1:])

    return [(template_name, template_name) for template_name in list(set(templates_list))]


class EmailTemplate(models.Model):
    """
    Model to store email template.
    """
    title = models.CharField(_(u'template'), max_length=255, choices=get_template_choices())
    subject = models.CharField(_(u'subject'), max_length=255, blank=True)
    content = models.TextField(_(u'content'))
    language = models.CharField(_(u'language'), max_length=10, choices=settings.LANGUAGES,
                                default=settings.LANGUAGE_CODE)
    created = models.DateTimeField(default=now)
    modified = models.DateTimeField(default=now)

    class Meta:
        unique_together = (('title', 'language'),)

    def __unicode__(self):
        return u'%s -> %s' % (self.title, self.language)



# coding=utf-8
from .email import EmailFromTemplate


def send_email(name, ctx_dict, send_to=None, subject=u'Subject', prefix='emailtemplates', **kwargs):
    """
    Shortcut function for EmailFromTemplate class

    @return: None
    """
    eft = EmailFromTemplate(prefix=prefix, name=name)
    eft.subject = subject
    eft.context = ctx_dict
    eft.get_object()
    eft.render_message()
    eft.send_email(send_to=send_to, **kwargs)



VERSION = (0, 8, 0)

# Dynamically calculate the version based on VERSION tuple
if len(VERSION) > 2 and VERSION[2] is not None:
    if isinstance(VERSION[2], int):
        str_version = "%s.%s.%s" % VERSION[:3]
    else:
        str_version = "%s.%s_%s" % VERSION[:3]
else:
    str_version = "%s.%s" % VERSION[:2]

__version__ = str_version



# coding=utf-8
from string import lower


class SubstringMatcher(object):
    """
    Class to be used with Mock() in order not to supply full content
    of the argument (e.g. for logger).
    Based on: http://www.michaelpollmeier.com/python-mock-how-to-assert-a-substring-of-logger-output/

    Usage with this class aliased to substr:
    email_logger.warning.assert_called_with(substr("Can't find EmailTemplate object in database"))

    This would do the same, but requires exactly the same argument content:
    email_logger.warning.assert_called_with("Can't find EmailTemplate object in database, using default file template.")
    """
    def __init__(self, containing):
        self.containing = lower(containing)

    def __eq__(self, other):
        return lower(other).find(self.containing) > -1

    def __unicode__(self):
        return 'a string containing "%s"' % self.containing

    def __str__(self):
        return unicode(self).encode('utf-8')

    __repr__ = __unicode__

substr = SubstringMatcher



# coding=utf-8
import os
import logging
from smtplib import SMTPException

from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import EmailMessage
from django.template import Template, Context
from django.template.base import TemplateDoesNotExist
from django.template.loader import get_template

from .models import now, EmailTemplate


logger = logging.getLogger(__name__)


class EmailFromTemplate(object):
    """
    EmailFromTemplate class tries to load template from database. If template object is not found,
    it tries to load template from templates/emailtemplates/{{prefix}} directory.
    This enables developers to create default templates for emails that are sent,
    and Site Admins to easily override the templates and provide email translations.

    Site Admins should know given template context.
    Site Admins should be familiar with Django Template System.
    """
    def __init__(self,
                 name="",
                 from_email=settings.DEFAULT_FROM_EMAIL,
                 language=settings.LANGUAGE_CODE,
                 subject="",
                 prefix="",
                 template_class=EmailTemplate
    ):
        """
        Class constructor

        @param name: template name
        @param from_email: sender email address, by default settings.DEFAULT_FROM_EMAIL
        @param language: email language, by default settings.LANGUAGE_CODE
        @param subject: subject of the email
        @param prefix: folder, where templates are stored (e.g. prefix 'email' and name 't.htm' will search email/t.htm)
        @param template_class: class, template objects will be retrieved from

        By default 'date' context variable is filled in.
        """
        self.from_email = from_email
        self.template_class = template_class
        self.subject = subject
        self.language = language
        self.name = name
        self.prefix = prefix

        self.template = None
        self.compiled_template = None  # for storing compiled template
        self.context = {'date': now()}  # default context
        self.sent = 0  # number of messages sent
        self.message = ""
        self.content_subtype = 'html'
        self._template_source = 'default'

    @property
    def template_source(self):
        """Source of the template. One of the following:
           * default
           * filesystem
           * database
        """
        return self._template_source

    def __get_path(self):
        return os.path.join(self.prefix, self.name)

    def __get_template_from_file(self):
        path = self.__get_path()
        try:
            self.compiled_template = get_template(path)
        except TemplateDoesNotExist:
            logger.warning("Can't find %s template in the filesystem, will use very default one." % path)
        else:
            self._template_source = 'filesystem'

    def get_object(self):
        language = self.language
        while True:
            try:
                tmp = self.template_class.objects.get(title=self.name, language=language)
                self.template = unicode(tmp.content)
                self.subject = unicode(tmp.subject)
            except ObjectDoesNotExist:
                logger.warning("Can't find EmailTemplate object in database, using default file template.")
                break
            except UnicodeError:
                logger.warning(
                    "Can't convert to unicode EmailTemplate object from database, using default file template.")
                break
            else:
                logger.debug(u"Got template %s from database" % self.name)
                self._template_source = 'database'
                return
        # fallback
        self.__get_template_from_file()

    def __compile_template(self):
        if not self.compiled_template:
            self.compiled_template = Template(self.template)

    def render_message(self):
        self.__compile_template()
        context = Context(self.context)
        self.message = self.compiled_template.render(context)

    def send_email(self, send_to, *args, **kwargs):
        """
        Sends email to recipient based on self object parameters.

        @param send_to: recipient email
        @param args: additional args passed to EmailMessage
        @param kwargs: kwargs passed to EmailMessage
        @return: number of sent messages
        """
        msg = EmailMessage(self.subject, self.message, self.from_email, send_to, *args, **kwargs)
        msg.content_subtype = self.content_subtype

        try:
            self.sent = msg.send()
        except SMTPException, e:
            logger.error(u'Problem sending email to %s: %s' % (send_to, e))

        return self.sent

    def send(self, to, *args, **kwargs):
        """This function does all the operations on eft object, that are necessary to send email.
           Usually one would use eft object like this:
                eft = EmailFromTemplate(name='sth/sth.html')
                eft.get_object()
                eft.render_message()
                eft.send_email(['email@example.com'])
                return eft.sent
        """
        self.get_object()
        self.render_message()
        self.send_email(to, *args, **kwargs)
        logger.info(u"Mail has been sent to: %s " % to)
        return self.sent



# coding=utf-8
from django.contrib import admin
from .forms import EmailTemplateAdminForm
from .models import EmailTemplate


class EmailTemplateAdmin(admin.ModelAdmin):
    """
    Admin view of EmailTemplate
    """
    list_display = ('title', 'language', 'subject',)
    list_display_links = ('title',)
    list_filter = ('title', 'language',)
    search_fields = ('title', 'subject')
    form = EmailTemplateAdminForm
    save_on_top = True
    save_as = True

admin.site.register(EmailTemplate, EmailTemplateAdmin)



# coding=utf-8
import logging

import os
from django import forms
from django.template.loaders import app_directories
from django.utils.translation import ugettext_lazy as _

from .models import EmailTemplate

logger = logging.getLogger(__name__)


class TemplateSourceLoader(app_directories.Loader):
    def get_source(self, template_name):
        source, origin = self.load_template_source(template_name)
        return source


class EmailTemplateAdminForm(forms.ModelForm):
    preview = forms.CharField(required=False,
                              label=_(u'default template preview'),
                              help_text=_(u'Preview of the default template is available after selecting a template '
                                          u'and saving changes (Save and continue editing)'))

    class Meta:
        model = EmailTemplate
        fields = [
            'title',
            'subject',
            'content',
            'language',
            'created',
            'modified',
        ]

    def __init__(self, *args, **kwargs):
        super(EmailTemplateAdminForm, self).__init__(*args, **kwargs)
        self.fields['preview'].widget = forms.Textarea(attrs={'readonly': 'readonly', 'rows': 30, 'cols': 120})
        if self.initial:
            loader = TemplateSourceLoader()
            try:
                self.fields['preview'].initial = loader.get_source(
                    os.path.join("emailtemplates", self.initial['title']))
            except Exception, e:
                logger.error('Load template error. Details: %s', e)
